package edu.ntnu.stud.chaosgame.controller.game;

import edu.ntnu.stud.chaosgame.model.data.Vector2D;
import edu.ntnu.stud.chaosgame.view.ChaosGameObserver;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;

/**
 * Class representing the chaos game.
 */
public class ChaosGame {

  /**
   * The canvas for this ChaosGame
   */
  private ChaosCanvas canvas;

  /**
   * The chaos game description.
   */
  private ChaosGameDescription description;

  /**
   * Observers monitoring this ChaosGame.
   */
 private ArrayList<ChaosGameObserver> observers;

  /**
   * The current point in the chaos game.
   */
  private Vector2D currentPoint;

  /**
   * The random number generator for the chaos game.
   */
  private Random random;

  /**
   * Number of transforms.
   */
  private int numOfTransforms;

  /**
   * Basic parameterised constructor.
   *
   * @param description the ChaosGameDescription.
   * @param canvas the ChaosCanvas.
   */
  public ChaosGame(ChaosGameDescription description, ChaosCanvas canvas) {
    this.canvas = canvas;
    this.description = description;
    this.random = new Random();
    this.numOfTransforms = description.getTransforms().size();
    this.currentPoint = new Vector2D(0, 0);
  }

  /**
   * Parameterised constructor with observers.
   *
   * @param description the ChaosGameDescription.
   * @param canvas the ChaosCanvas.
   * @param observers the ChaosGameObservers to be added.
   */
  public ChaosGame(ChaosGameDescription description, ChaosCanvas canvas, ChaosGameObserver... observers) {
    this.canvas = canvas;
    this.description = description;
    this.random = new Random();
    this.numOfTransforms = description.getTransforms().size();
    this.currentPoint = new Vector2D(0, 0);
    this.observers.addAll(Arrays.asList(observers));
  }


  /**
   * Get the canvas of this chaos game.
   *
   * @return the canvas.
   */
  public ChaosCanvas getCanvas() { return this.canvas; }

  /**
   * Get the current point in the chaos game.
   *
   * @return the point.
   */
  public Vector2D getCurrentPoint() { return this.currentPoint; }


  /**
   * Notify the observers that a change has occurred in the ChaosGame.
   */
  public void notifyObservers() {
    for (ChaosGameObserver observer : this.observers) {
      observer.update(this);
    }
  }

  /**
   * Add an observer to the ChaosGame.
   *
   * @param observer the observer to add.
   */
  public void addObserver(ChaosGameObserver observer) {
    this.observers.add(observer);
  }

  /**
   * Run the chaos game for n iterations.
   *
   * @param n the number of iterations.
   */
  public void runSteps(int n) {
    for (int i = 0; i < n; i++) {
      int j = this.random.nextInt(0, this.numOfTransforms);
      //System.out.println("Before transform: " + currentPoint.getX0() + " " + currentPoint.getX1());

      this.currentPoint = this.description.getTransforms().get(j).transform(currentPoint);
      //System.out.println("After transform: " + currentPoint.getX0() + " " + currentPoint.getX1());
      this.canvas.putPixel(currentPoint);
    }
    //this.canvas.printCanvas();
  }

}
